part of 'tmap_view.widget.dart';

/// marker点击事件回调签名 输入[Marker]对象, 返回`是否已消耗事件`, 如果true则不再弹窗, 如果false则继续弹窗
typedef Future<bool> OnMarkerClicked(Marker marker);

/// 地图点击事件回调签名
typedef Future<void> OnMapClicked(LatLng latLng);

/// 地图移动事件回调签名
typedef Future<void> OnMapMove(MapMove move);

/// 位置移动事件回调签名
//typedef Future<void> OnLocationChange(MapLocation move);

/// Marker拖动回调签名
typedef Future<void> OnMarkerDrag(Marker marker);

/// ios请求权限回调签名
typedef Future<void> _OnRequireAlwaysAuth(CLLocationManager manager);

/// 地图截屏回调签名
typedef Future<void> OnScreenShot(Uint8List imageData);

/// 海量点点击回调签名
typedef Future<void> OnMultiPointClicked(
  String id,
  String title,
  String snippet,
  String object,
);

class TmapController with WidgetsBindingObserver {
  /// Android构造器
  TmapController.android(this._androidController, this._state) {
    WidgetsBinding.instance.addObserver(this);
  }

  /// iOS构造器
  TmapController.ios(this._iosController, this._state) {
    WidgetsBinding.instance.addObserver(this);
  }

  com_tencent_tencentmap_mapsdk_maps_TextureMapView _androidController;
  QMapView _iosController;

  _TmapViewState _state;

  final _iosMapDelegate = _IOSMapDelegate();
  final _androidMapDelegate = _AndroidMapDelegate();

  /// 是否显示室内地图
  Future<void> showIndoorMap(bool show) async {
    await platform(
      android: (pool) async {
        final map = await _androidController.getMap();
        await map.setIndoorEnabled(show);

        pool..add(map);
      },
      ios: (pool) async {
        await _iosController.setIndoorEnabled(show);
      },
    );
  }

  /// 选择显示图层
  Future<void> setMapType(MapType mapType) async {
    await platform(
      android: (pool) async {
        final map = await _androidController.getMap();
        switch (mapType) {
          case MapType.Standard:
            await map.setMapType(1000);
            await map.setSatelliteEnabled(false);
            break;
          case MapType.Satellite:
            await map.setSatelliteEnabled(true);
            break;
          case MapType.Night:
            await map.setMapType(1008);
            await map.setSatelliteEnabled(false);
            break;
        }

        pool..add(map);
      },
      ios: (pool) async {
        switch (mapType) {
          case MapType.Standard:
            await _iosController.set_mapType(QMapType.QMapTypeStandard);
            break;
          case MapType.Satellite:
            await _iosController.set_mapType(QMapType.QMapTypeSatellite);
            break;
          case MapType.Night:
            await _iosController.set_mapType(QMapType.QMapTypeDark);
            break;
        }
      },
    );
  }

  /// 显示路况信息
  Future<void> showTraffic(bool enable) async {
    await platform(
      android: (pool) async {
        final map = await _androidController.getMap();
        await map.setTrafficEnabled(enable);

        pool..add(map);
      },
      ios: (pool) async {
        await _iosController.set_showsTraffic(enable);
      },
    );
  }

  /// 显示缩放控件
  Future<void> showZoomControl(bool enable) async {
    await platform(
      android: (pool) async {
        final map = await _androidController.getMap();
        final uiSetting = await map.getUiSettings();
        await uiSetting.setZoomControlsEnabled(enable);

        pool..add(map)..add(uiSetting);
      },
      ios: (pool) async {
        print('ios端不支持显示缩放控件');
      },
    );
  }

  /// 显示指南针
  Future<void> showCompass(bool enable) async {
    await platform(
      android: (pool) async {
        final map = await _androidController.getMap();
        final uiSetting = await map.getUiSettings();
        await uiSetting.setCompassEnabled(enable);

        pool..add(map)..add(uiSetting);
      },
      ios: (pool) async {
        await _iosController.set_showsCompass(enable);
      },
    );
  }

  /// 显示定位按钮
  Future<void> showLocateControl(bool enable) async {
    await platform(
      android: (pool) async {
        final map = await _androidController.getMap();
        final uiSetting = await map.getUiSettings();
        await uiSetting.setMyLocationButtonEnabled(enable);

        pool..add(map)..add(uiSetting);
      },
      ios: (pool) async {
        print('ios端不支持显示定位按钮');
      },
    );
  }

  /// 显示比例尺控件
  Future<void> showScaleControl(bool enable) async {
    await platform(
      android: (pool) async {
        final map = await _androidController.getMap();
        final uiSetting = await map.getUiSettings();
        await uiSetting.setScaleViewEnabled(enable);

        pool..add(map)..add(uiSetting);
      },
      ios: (pool) async {
        await _iosController.set_showsScale(enable);
      },
    );
  }

  /// 缩放手势使能
  Future<void> setZoomGesturesEnabled(bool enable) async {
    await platform(
      android: (pool) async {
        final map = await _androidController.getMap();
        final uiSetting = await map.getUiSettings();
        await uiSetting.setZoomGesturesEnabled(enable);

        pool..add(map)..add(uiSetting);
      },
      ios: (pool) async {
        await _iosController.set_zoomEnabled(enable);
      },
    );
  }

  /// 滑动手势使能
  Future<void> setScrollGesturesEnabled(bool enable) async {
    await platform(
      android: (pool) async {
        final map = await _androidController.getMap();
        final uiSetting = await map.getUiSettings();
        await uiSetting.setScrollGesturesEnabled(enable);

        pool..add(map)..add(uiSetting);
      },
      ios: (pool) async {
        await _iosController.set_scrollEnabled(enable);
      },
    );
  }

  /// 旋转手势使能
  Future<void> setRotateGesturesEnabled(bool enable) async {
    await platform(
      android: (pool) async {
        final map = await _androidController.getMap();
        final uiSetting = await map.getUiSettings();
        await uiSetting.setRotateGesturesEnabled(enable);

        pool..add(map)..add(uiSetting);
      },
      ios: (pool) async {
        await _iosController.set_rotateEnabled(enable);
      },
    );
  }

  /// 倾斜手势使能
  Future<void> setTiltGesturesEnabled(bool enable) async {
    await platform(
      android: (pool) async {
        final map = await _androidController.getMap();
        final uiSetting = await map.getUiSettings();
        await uiSetting.setTiltGesturesEnabled(enable);

        pool..add(map)..add(uiSetting);
      },
      ios: (pool) async {
        await _iosController.set_rotateEnabled(enable);
      },
    );
  }

  /// 所有手势使能
  Future<void> setAllGesturesEnabled(bool enable) async {
    await platform(
      android: (pool) async {
        final map = await _androidController.getMap();
        final uiSetting = await map.getUiSettings();
        await uiSetting.setAllGesturesEnabled(enable);

        pool..add(map)..add(uiSetting);
      },
      ios: (pool) async {
        await _iosController.set_zoomEnabled(enable);
        await _iosController.set_scrollEnabled(enable);
        await _iosController.set_rotateEnabled(enable);
        await _iosController.set_rotateEnabled(enable);
      },
    );
  }

  /// 添加marker
  Future<Marker> addMarker(MarkerOption option) async {
    assert(option != null);

    final latitude = option.latLng.latitude;
    final longitude = option.latLng.longitude;
    return platform(
      android: (pool) async {
        final map = await _androidController.getMap();
        // marker配置
        final markerOption =
            await com_tencent_tencentmap_mapsdk_maps_model_MarkerOptions
                .create__();

        // marker经纬度
        final latLng = await com_tencent_tencentmap_mapsdk_maps_model_LatLng
            .create__double__double(latitude, longitude);

        // 设置marker经纬度
        await markerOption.position(latLng);
        // 设置marker标题
        if (option.title != null) {
          await markerOption.title(option.title);
        }
        // 设置marker副标题
        if (option.snippet != null) {
          await markerOption.snippet(option.snippet);
        }
        // 设置marker图标
        // 普通图片
        if (option.iconUri != null && option.imageConfig != null) {
          Uint8List iconData =
              await uri2ImageData(option.imageConfig, option.iconUri);

          final bitmap = await android_graphics_Bitmap.create(iconData);
          final icon =
              await com_tencent_tencentmap_mapsdk_maps_model_BitmapDescriptorFactory
                  .fromBitmap__android_graphics_Bitmap(bitmap);
          await markerOption.icon(icon);

          pool..add(bitmap)..add(icon);
        }
        // 是否可拖拽
        if (option.draggable != null) {
          await markerOption.draggable(option.draggable);
        }
        // 锚点 默认在中间底部是最合理的
        await markerOption.anchor(option.anchorU ?? 0.5, option.anchorV ?? 0);
        // 是否可见
        await markerOption.visible(option.visible);

        final marker = await map.addMarker(markerOption);

        // 是否允许弹窗
        if (option.infoWindowEnabled != null) {
          await marker.setInfoWindowEnable(option.infoWindowEnabled);
        }

        // 自定义数据
        if (option.object != null) {
          await marker.setTag(option.object);
        }

        pool..add(map)..add(markerOption);

        return Marker.android(marker);
      },
      ios: (pool) async {
        await _iosController.set_delegate(
          _iosMapDelegate
            .._iosController = _iosController
            .._annotationViewCompleter = Completer(),
        );

        // 创建marker
        final annotation = await QPointAnnotation.create__();

        final coordinate =
            await CLLocationCoordinate2D.create(latitude, longitude);

        // 设置经纬度
        await annotation.set_coordinate(coordinate);

        // 设置标题
        if (option.title != null) {
          await annotation.set_title(option.title);
        }
        // 设置副标题
        if (option.snippet != null) {
          await annotation.set_subtitle(option.snippet);
        }
        // 设置图片
        // 普通图片
        if (option.iconUri != null && option.imageConfig != null) {
          Uint8List iconData =
              await uri2ImageData(option.imageConfig, option.iconUri);

          final icon = await UIImage.create(iconData);

          // 由于ios端的icon参数在回调中设置, 需要添加属性来实现
          await annotation.addProperty__(1, icon);

          pool..add(icon);
        }
        // 是否可拖拽
        if (option.draggable != null) {
          await annotation.addJsonableProperty__(2, option.draggable);
        }
        // 是否允许弹窗
        if (option.infoWindowEnabled != null) {
          annotation.addJsonableProperty__(4, option.infoWindowEnabled);
        }
        // 锚点
        if (option.anchorU != null || option.anchorV != null) {
          await annotation.addJsonableProperty__(5, option.anchorU);
          await annotation.addJsonableProperty__(6, option.anchorV);
        }
        // 自定义数据
        if (option.object != null) {
          await annotation.addJsonableProperty__(7, option.object);
        }
        // 是否可见
        await annotation.addJsonableProperty__(10, option.visible);

        // 添加marker
        await _iosController.addAnnotation(annotation);

        // 等待添加完成 获取对应的view
        final annotationViewList =
            await _iosMapDelegate._annotationViewCompleter.future;
        // pointAnnotation不释放, 还有用
        pool.add(coordinate);

        return Marker.ios(annotation, annotationViewList[0], _iosController);
      },
    );
  }

  /// 显示我的位置
  Future<void> showMyLocation(MyLocationOption option) async {
    assert(option != null);
    await platform(
      android: (pool) async {
        final map = await _androidController.getMap();
        await map.setLocationSource(_androidMapDelegate);
        final locationStyle =
            await com_tencent_tencentmap_mapsdk_maps_model_MyLocationStyle
                .create__();
        await map.setMyLocationEnabled(option.show);

        if (option.show) {
          switch (option.myLocationType) {
            case MyLocationType.Locate:
              await locationStyle.myLocationType(
                com_tencent_tencentmap_mapsdk_maps_model_MyLocationStyle
                    .LOCATION_TYPE_LOCATION_ROTATE,
              );
              break;
            case MyLocationType.Follow:
              await locationStyle.myLocationType(
                com_tencent_tencentmap_mapsdk_maps_model_MyLocationStyle
                    .LOCATION_TYPE_FOLLOW_NO_CENTER,
              );
              break;
            case MyLocationType.Rotate:
              await locationStyle.myLocationType(
                com_tencent_tencentmap_mapsdk_maps_model_MyLocationStyle
                    .LOCATION_TYPE_LOCATION_ROTATE,
              );
              break;
          }

          // 定位图标
          if (option.iconUri != null) {
            final imageData = await uri2ImageData(
              option.imageConfiguration,
              option.iconUri,
              package: option.package,
            );
            final bitmap = await android_graphics_Bitmap.create(imageData);
            final bitmapDescriptor =
                await com_tencent_tencentmap_mapsdk_maps_model_BitmapDescriptorFactory
                    .fromBitmap__android_graphics_Bitmap(bitmap);
            await locationStyle.icon(bitmapDescriptor);
          }
          // 边框颜色
          if (option.strokeColor != null) {
            await locationStyle
                .strokeColor(Int32List.fromList([option.strokeColor.value])[0]);
          }
          // 填充颜色
          if (option.fillColor != null) {
            await locationStyle
                .fillColor(Int32List.fromList([option.fillColor.value])[0]);
          }
          // 边框宽度
          if (option.strokeWidth != null) {
            await locationStyle.strokeWidth(option.strokeWidth.toInt());
          }

          // 设置样式
          await map.setMyLocationStyle(locationStyle);
        }

        pool..add(map)..add(locationStyle);
      },
      ios: (pool) async {
        await _iosController.set_showsUserLocation(option.show);
        if (option.show) {
          switch (option.myLocationType) {
            // ios端没有只定位一次的选项, 所以这里要模拟一下这个效果
            // 1. 先设置不跟踪位置, 防止定位marker来回跳
            // 2. 获取当前用户位置
            // 3. 把当前地图中心点设置为用户位置
            case MyLocationType.Locate:
              await _iosController.setUserTrackingMode_animated(
                QUserTrackingMode.QUserTrackingModeNone,
                true,
              );
              final myLocation = await _iosController.get_userLocation();
              final location = await myLocation.get_location();
              await _iosController.setCenterCoordinate_animated(
                await location.coordinate,
                false,
              );
              break;
            case MyLocationType.Follow:
              await _iosController.setUserTrackingMode_animated(
                QUserTrackingMode.QUserTrackingModeFollow,
                true,
              );
              break;
            case MyLocationType.Rotate:
              await _iosController.setUserTrackingMode_animated(
                QUserTrackingMode.QUserTrackingModeFollowWithHeading,
                true,
              );
              break;
          }

          final style = await QUserLocationPresentation.create__();

          // 定位图标
          if (option.iconUri != null) {
            final imageData = await uri2ImageData(
              option.imageConfiguration,
              option.iconUri,
              package: option.package,
            );
            final bitmap = await UIImage.create(imageData);
            await style.set_icon(bitmap);
          }
          // 填充颜色
          if (option.fillColor != null) {
            final color = await UIColor.create(option.fillColor);
            await style.set_circleFillColor(color);
          }

          await _iosController.configureUserLocationPresentation(style);
        }
      },
    );
  }

  /// 设置地图中心点
  ///
  /// [lat]纬度, [lng]经度, [zoomLevel]缩放等级, [bearing]地图选择角度, [tilt]倾斜角
  Future<void> setCenterCoordinate(
    LatLng coordinate, {
    double zoomLevel,
    double bearing,
    double tilt,
    bool animated = true,
  }) async {
    assert(
      zoomLevel == null || (zoomLevel >= 3 && zoomLevel <= 19),
      '缩放范围为3-19',
    );
    final lat = coordinate.latitude;
    final lng = coordinate.longitude;
    await platform(
      android: (pool) async {
        final map = await _androidController.getMap();

        final latLng = await com_tencent_tencentmap_mapsdk_maps_model_LatLng
            .create__double__double(lat, lng);

        final camera = await map.getCameraPosition();
        final finalZoomLevel = zoomLevel ?? await camera.get_zoom();
        final finalBearing = bearing ?? await camera.get_bearing();
        final finalTilt = tilt ?? await camera.get_tilt();
        final cameraPosition =
            await com_tencent_tencentmap_mapsdk_maps_model_CameraPosition
                .create__com_tencent_tencentmap_mapsdk_maps_model_LatLng__float__float__float(
          latLng,
          finalZoomLevel,
          finalTilt,
          finalBearing,
        );

        final cameraUpdate =
            await com_tencent_tencentmap_mapsdk_maps_CameraUpdateFactory
                .newCameraPosition(cameraPosition);

        if (animated) {
          await map
              .animateCamera__com_tencent_tencentmap_mapsdk_maps_CameraUpdate(
            cameraUpdate,
          );
        } else {
          await map.moveCamera(cameraUpdate);
        }

        pool..add(map)..add(latLng)..add(cameraUpdate);
      },
      ios: (pool) async {
        final latLng = await CLLocationCoordinate2D.create(lat, lng);
        if (zoomLevel != null) {
          await _iosController.setZoomLevel_animated(zoomLevel, animated);
        }
        if (bearing != null) {
          await _iosController.set_rotation(bearing);
        }
        if (tilt != null) {
          // TODO 不知道是哪个参数
//          await _iosController.set(tilt);
        }
        await _iosController.setCenterCoordinate_animated(latLng, animated);

        pool..add(latLng);
      },
    );
  }

  /// 把marker列表从地图上移除
  Future<void> clearMarkers(List<Marker> markers) async {
    await platform(
      android: (pool) async {
        final markerBatch = markers.map((it) => it.androidModel).toList();
        await markerBatch.remove_batch();
      },
      ios: (pool) async {
        final markerBatch = markers.map((it) => it.iosModel).toList();
        await _iosController.removeAnnotations(markerBatch);
      },
    );
  }

  /// 清除地图上所有覆盖物
  ///
  /// 根据[keepMyLocation]区分是否保留我的位置的marker
  Future<void> clear({bool keepMyLocation = true}) async {
    await platform(
      android: (pool) async {
        final map = await _androidController.getMap();
        if (keepMyLocation)
          await map.clearAllOverlays();
        else
          await map.clear();

        pool.add(map);
      },
      ios: (pool) async {
        final markers = await _iosController.get_annotations();
        final overlays = await _iosController.get_overlays();
        await _iosController.removeAnnotations(markers);
        await _iosController.removeOverlays(overlays);

        pool..addAll(markers.cast<Ref>())..addAll(overlays.cast<Ref>());
      },
    );
  }

  /// 添加折线
  ///
  /// 可配置参数详见[PolylineOption]
  Future<Polyline> addPolyline(PolylineOption option) {
    assert(option != null);

    final latitudeBatch = option.latLngList.map((e) => e.latitude).toList();
    final longitudeBatch = option.latLngList.map((e) => e.longitude).toList();

    return platform(
      android: (pool) async {
        final map = await _androidController.getMap();

        // 构造折线点
        List<com_tencent_tencentmap_mapsdk_maps_model_LatLng> latLngList =
            await com_tencent_tencentmap_mapsdk_maps_model_LatLng
                .create_batch__double__double(
          latitudeBatch,
          longitudeBatch,
        );

        // 构造折线参数
        final polylineOptions =
            await com_tencent_tencentmap_mapsdk_maps_model_PolylineOptions
                .create__();

        // 添加经纬度列表
        await polylineOptions.addAll(latLngList);
        if (option.width != null) {
          await polylineOptions.width(option.width);
        }
        // 颜色
        if (option.strokeColor != null) {
          await polylineOptions
              .color(Int32List.fromList([option.strokeColor.value])[0]);
        }
        // 设置参数
        final polyline = await map.addPolyline(polylineOptions);

        pool
          ..add(map)
          ..add(polylineOptions)
          ..addAll(latLngList);

        return Polyline.android(polyline);
      },
      ios: (pool) async {
        await _iosController.set_delegate(_iosMapDelegate);

        // 构造折线点
        List<CLLocationCoordinate2D> latLngList =
            await CLLocationCoordinate2D.create_batch(
                latitudeBatch, longitudeBatch);

        // 构造折线参数
        final polyline = await QPolyline.polylineWithCoordinates_count(
            latLngList, latLngList.length);

        if (option.width != null) {
          final pixelRatio = MediaQuery.of(_state.context).devicePixelRatio;
          polyline.addJsonableProperty__(1, option.width / pixelRatio);
        }
        // 颜色
        if (option.strokeColor != null) {
          polyline.addJsonableProperty__(2, option.strokeColor.value);
        }

        // 设置参数
        await _iosController.addOverlay(polyline);

        pool..addAll(latLngList);

        return Polyline.ios(polyline, _iosController);
      },
    );
  }

  /// 添加多边形
  ///
  /// 在点[points]的位置添加线, 可以设置宽度[width]和颜色[strokeColor]
  Future<Polygon> addPolygon(PolygonOption option) {
    assert(option != null, 'option不能为null');

    final latitudeBatch = option.latLngList.map((e) => e.latitude).toList();
    final longitudeBatch = option.latLngList.map((e) => e.longitude).toList();

    return platform(
      android: (pool) async {
        final map = await _androidController.getMap();

        // 构造折线点
        List<com_tencent_tencentmap_mapsdk_maps_model_LatLng> latLngList =
            await com_tencent_tencentmap_mapsdk_maps_model_LatLng
                .create_batch__double__double(
          latitudeBatch,
          longitudeBatch,
        );

        // 构造参数
        final polygonOptions =
            await com_tencent_tencentmap_mapsdk_maps_model_PolygonOptions
                .create__();

        // 添加参数
        await polygonOptions.addAll(latLngList);
        // 宽度
        if (option.width != null) {
          await polygonOptions.strokeWidth(option.width);
        }
        // 边框颜色
        if (option.strokeColor != null) {
          await polygonOptions
              .strokeColor(Int32List.fromList([option.strokeColor.value])[0]);
        }
        // 填充颜色
        if (option.fillColor != null) {
          await polygonOptions
              .fillColor(Int32List.fromList([option.fillColor.value])[0]);
        }

        // 设置参数
        final polygon = await map.addPolygon(polygonOptions);

        pool
          ..add(map)
          ..add(polygonOptions)
          ..addAll(latLngList);

        return Polygon.android(polygon);
      },
      ios: (pool) async {
        await _iosController.set_delegate(_iosMapDelegate);

        // 构造折线点
        List<CLLocationCoordinate2D> latLngList =
            await CLLocationCoordinate2D.create_batch(
                latitudeBatch, longitudeBatch);

        // 构造折线参数
        final polygon = await QPolygon.polygonWithCoordinates_count(
            latLngList, latLngList.length);

        if (option.width != null) {
          final pixelRatio = MediaQuery.of(_state.context).devicePixelRatio;
          polygon.addJsonableProperty__(1, option.width / pixelRatio);
        }
        if (option.strokeColor != null) {
          polygon.addJsonableProperty__(2, option.strokeColor.value);
        }
        if (option.fillColor != null) {
          polygon.addJsonableProperty__(3, option.fillColor.value);
        }

        // 设置参数
        await _iosController.addOverlay(polygon);

        pool.addAll(latLngList);

        return Polygon.ios(polygon, _iosController);
      },
    );
  }

  /// 添加圆
  ///
  /// 在点[points]的位置添加线, 可以设置宽度[width]和颜色[strokeColor]
  Future<Circle> addCircle(CircleOption option) {
    return platform(
      android: (pool) async {
        final map = await _androidController.getMap();

        // 构造点
        final latLng = await com_tencent_tencentmap_mapsdk_maps_model_LatLng
            .create__double__double(
          option.center.latitude,
          option.center.longitude,
        );

        // 构造参数
        final circleOptions =
            await com_tencent_tencentmap_mapsdk_maps_model_CircleOptions
                .create__();

        // 中心点
        await circleOptions.center(latLng);
        // 半径
        await circleOptions.radius(option.radius);
        // 宽度
        if (option.width != null) {
          await circleOptions.strokeWidth(option.width);
        }
        // 边框颜色
        if (option.strokeColor != null) {
          await circleOptions
              .strokeColor(Int32List.fromList([option.strokeColor.value])[0]);
        }
        // 填充颜色
        if (option.fillColor != null) {
          await circleOptions
              .fillColor(Int32List.fromList([option.fillColor.value])[0]);
        }

        // 设置参数
        final circle = await map.addCircle(circleOptions);

        pool..add(map)..add(circleOptions)..add(latLng);

        return Circle.android(circle);
      },
      ios: (pool) async {
        await _iosController.set_delegate(_iosMapDelegate);

        final latLng = await CLLocationCoordinate2D.create(
          option.center.latitude,
          option.center.longitude,
        );

        // 参数
        final circle = await QCircle.circleWithCenterCoordinate_radius(
          latLng,
          option.radius,
        );

        if (option.width != null) {
          final pixelRatio = MediaQuery.of(_state.context).devicePixelRatio;
          circle.addJsonableProperty__(1, option.width / pixelRatio);
        }
        if (option.strokeColor != null) {
          circle.addJsonableProperty__(2, option.strokeColor.value);
        }
        if (option.fillColor != null) {
          circle.addJsonableProperty__(3, option.fillColor.value);
        }

        // 设置参数
        await _iosController.addOverlay(circle);

        return Circle.ios(circle, _iosController);
      },
    );
  }

  /// 设置marker点击监听事件
  Future<void> setMarkerClickedListener(OnMarkerClicked onMarkerClicked) async {
    await platform(
      android: (pool) async {
        final map = await _androidController.getMap();

        await map.setOnMarkerClickListener(
            _androidMapDelegate.._onMarkerClicked = onMarkerClicked);

        pool..add(map);
      },
      ios: (pool) async {
        await _iosController
            .set_delegate(_iosMapDelegate.._onMarkerClicked = onMarkerClicked);
      },
    );
  }

  Future<void> dispose() async {
    await _androidController?.onPause();
    await _androidController?.onStop();
    await _androidController?.onDestroy();

    WidgetsBinding.instance.removeObserver(this);
  }

  @override
  void didChangeAppLifecycleState(AppLifecycleState state) {
    super.didChangeAppLifecycleState(state);
    debugPrint('didChangeAppLifecycleState: $state');
    // 因为这里的生命周期其实已经是App的生命周期了, 所以除了这里还需要在dispose里释放资源
    switch (state) {
      case AppLifecycleState.resumed:
        _androidController?.onResume();
        break;
      case AppLifecycleState.inactive:
        break;
      case AppLifecycleState.paused:
        _androidController?.onPause();
        break;
      case AppLifecycleState.detached:
        _androidController?.onDestroy();
        break;
    }
  }
}
